@@ -1,4 +1,9 @@
-"""Import a set of non-hipscat files using dask for parallelization"""
+"""Import a set of non-hipscat files using dask for parallelization
+
+Methods in this file set up a dask pipeline using futures. 
+The actual logic of the map reduce is in the `map_reduce.py` file.
+"""
+
 
 import hipscat.io.write_metadata as io
 import numpy as np
@@ -7,17 +12,27 @@ from hipscat import pixel_math
 from tqdm import tqdm
 
 import hipscat_import.map_reduce as mr
+import hipscat_import.resume_files as resume
 from hipscat_import.arguments import ImportArguments
 
 
 def _map_pixels(args, client):
     """Generate a raw histogram of object counts in each healpix pixel"""
 
+    raw_histogram = resume.read_histogram(args.tmp_path, args.highest_healpix_order)
+    if resume.is_mapping_done(args.tmp_path):
+        return raw_histogram
+
+    mapped_paths = set(resume.read_mapping_keys(args.tmp_path))
+
     futures = []
     for i, file_path in enumerate(args.input_paths):
+        if file_path in mapped_paths:
+            continue
         futures.append(
             client.submit(
                 mr.map_to_pixels,
+                key=file_path,
                 input_file=file_path,
                 file_reader=args.file_reader,
                 filter_function=args.filter_function,
@@ -25,45 +40,61 @@ def _map_pixels(args, client):
                 ra_column=args.ra_column,
                 dec_column=args.dec_column,
                 shard_suffix=i,
-                cache_path=None if args.debug_stats_only else args.tmp_dir,
+                cache_path=None if args.debug_stats_only else args.tmp_path,
             )
         )
 
-    raw_histogram = pixel_math.empty_histogram(args.highest_healpix_order)
-    for _, result in tqdm(
+    for future, result in tqdm(
         as_completed(futures, with_results=True),
         desc="Mapping  ",
         total=len(futures),
         disable=(not args.progress_bar),
     ):
         raw_histogram = np.add(raw_histogram, result)
+        resume.write_mapping_start_key(args.tmp_path, future.key)
+        resume.write_histogram(args.tmp_path, raw_histogram)
+        resume.write_mapping_done_key(args.tmp_path, future.key)
+    resume.set_mapping_done(args.tmp_path)
     return raw_histogram
 
 
 def _reduce_pixels(args, destination_pixel_map, client):
     """Loop over destination pixels and merge into parquet files"""
 
+    if resume.is_reducing_done(args.tmp_path):
+        return
+
+    reduced_keys = set(resume.read_reducing_keys(args.tmp_path))
+
     futures = []
     for destination_pixel, source_pixels in destination_pixel_map.items():
+        destination_pixel_key = f"{destination_pixel[0]}_{destination_pixel[1]}"
+        if destination_pixel_key in reduced_keys:
+            continue
         futures.append(
             client.submit(
                 mr.reduce_pixel_shards,
-                cache_path=args.tmp_dir,
+                key=destination_pixel_key,
+                cache_path=args.tmp_path,
                 origin_pixel_numbers=source_pixels,
                 destination_pixel_order=destination_pixel[0],
                 destination_pixel_number=destination_pixel[1],
                 destination_pixel_size=destination_pixel[2],
                 output_path=args.catalog_path,
+                ra_column=args.ra_column,
+                dec_column=args.dec_column,
                 id_column=args.id_column,
+                add_hipscat_index=args.add_hipscat_index,
             )
         )
-    for _ in tqdm(
+    for future in tqdm(
         as_completed(futures),
         desc="Reducing ",
         total=len(futures),
         disable=(not args.progress_bar),
     ):
-        pass
+        resume.write_reducing_key(args.tmp_path, future.key)
+    resume.set_reducing_done(args.tmp_path)
 
 
 def _validate_args(args):
@@ -81,7 +112,7 @@ def run(args):
         local_directory=args.dask_tmp,
         n_workers=args.dask_n_workers,
         threads_per_worker=args.dask_threads_per_worker,
-    ) as client:
+    ) as client:  # pragma: no cover
         run_with_client(args, client)
 
 
@@ -105,11 +136,18 @@ def run_with_client(args, client):
         _reduce_pixels(args, destination_pixel_map, client)
 
     # All done - write out the metadata
-    step_progress = tqdm(total=3, desc="Finishing", disable=not args.progress_bar)
-    io.write_legacy_metadata(args, raw_histogram, pixel_map)
+    step_progress = tqdm(total=6, desc="Finishing", disable=not args.progress_bar)
+    io.write_provenance_info(args.to_catalog_parameters(), args.provenance_info())
     step_progress.update(1)
-    io.write_catalog_info(args, raw_histogram)
+    io.write_catalog_info(args.to_catalog_parameters(), raw_histogram)
     step_progress.update(1)
-    io.write_partition_info(args, destination_pixel_map)
+    if not args.debug_stats_only:
+        io.write_parquet_metadata(args.catalog_path)
+    step_progress.update(1)
+    io.write_fits_map(args.catalog_path, raw_histogram)
+    step_progress.update(1)
+    io.write_partition_info(args.to_catalog_parameters(), destination_pixel_map)
+    step_progress.update(1)
+    resume.clean_resume_files(args.tmp_path)
     step_progress.update(1)
     step_progress.close()
